@@ -18,7 +18,7 @@ from calibre.ebooks.docx.container import DOCX, fromstring
 from calibre.ebooks.docx.names import XML, generate_anchor
 from calibre.ebooks.docx.styles import Styles, inherit, PageProperties
 from calibre.ebooks.docx.numbering import Numbering
-from calibre.ebooks.docx.fonts import Fonts
+from calibre.ebooks.docx.fonts import Fonts, is_symbol_font, map_symbol_text
 from calibre.ebooks.docx.images import Images
 from calibre.ebooks.docx.tables import Tables
 from calibre.ebooks.docx.footnotes import Footnotes
@@ -32,15 +32,22 @@ from calibre.utils.localization import canonicalize_lang, lang_as_iso639_1
 
 NBSP = '\xa0'
 
+
 class Text:
 
     def __init__(self, elem, attr, buf):
         self.elem, self.attr, self.buf = elem, attr, buf
+        self.elems = [self.elem]
 
     def add_elem(self, elem):
+        self.elems.append(elem)
         setattr(self.elem, self.attr, ''.join(self.buf))
         self.elem, self.attr, self.buf = elem, 'tail', []
 
+    def __iter__(self):
+        return iter(self.elems)
+
+
 def html_lang(docx_lang):
     lang = canonicalize_lang(docx_lang)
     if lang and lang != 'und':
@@ -48,6 +55,7 @@ def html_lang(docx_lang):
         if lang:
             return lang
 
+
 class Convert(object):
 
     def __init__(self, path_or_stream, dest_dir=None, log=None, detect_cover=True, notes_text=None, notes_nopb=False, nosupsub=False):
@@ -112,6 +120,7 @@ class Convert(object):
         self.log.debug('Converting Word markup to HTML')
 
         self.read_page_properties(doc)
+        self.resolve_alternate_content(doc)
         self.current_rels = relationships_by_id
         for wp, page_properties in self.page_map.iteritems():
             self.current_page = page_properties
@@ -130,14 +139,14 @@ class Convert(object):
         notes_header = None
         orig_rid_map = self.images.rid_map
         if self.footnotes.has_notes:
-            dl = DL()
-            dl.set('class', 'notes')
             self.body.append(H1(self.notes_text))
             notes_header = self.body[-1]
             notes_header.set('class', 'notes-header')
-            self.body.append(dl)
             for anchor, text, note in self.footnotes:
-                dl.append(DT('[', A('←' + text, href='#back_%s' % anchor, title=text), id=anchor))
+                dl = DL(id=anchor)
+                dl.set('class', 'footnote')
+                self.body.append(dl)
+                dl.append(DT('[', A('←' + text, href='#back_%s' % anchor, title=text)))
                 dl[-1][0].tail = ']'
                 dl.append(DD())
                 paras = []
@@ -259,6 +268,17 @@ class Convert(object):
             for x in current:
                 self.page_map[x] = pr
 
+    def resolve_alternate_content(self, doc):
+        # For proprietary extensions in Word documents use the fallback, spec
+        # compliant form
+        # See https://wiki.openoffice.org/wiki/OOXML/Markup_Compatibility_and_Extensibility
+        for ac in self.namespace.descendants(doc, 'mc:AlternateContent'):
+            choices = self.namespace.XPath('./mc:Choice')(ac)
+            fallbacks = self.namespace.XPath('./mc:Fallback')(ac)
+            if fallbacks:
+                for choice in choices:
+                    ac.remove(choice)
+
     def read_styles(self, relationships_by_type):
 
         def get_name(rtype, defname):
@@ -268,6 +288,8 @@ class Convert(object):
                 cname[-1] = defname
                 if self.docx.exists('/'.join(cname)):
                     name = name
+            if name and name.startswith('word/word') and not self.docx.exists(name):
+                name = name.partition('/')[2]
             return name
 
         nname = get_name(self.namespace.names['NUMBERING'], 'numbering.xml')
@@ -328,6 +350,7 @@ class Convert(object):
             else:
                 self.theme(fromstring(raw))
 
+        styles_loaded = False
         if sname is not None:
             try:
                 raw = self.docx.read(sname)
@@ -335,6 +358,9 @@ class Convert(object):
                 self.log.warn('Styles %s do not exist' % sname)
             else:
                 self.styles(fromstring(raw), fonts, self.theme)
+                styles_loaded = True
+        if not styles_loaded:
+            self.styles(None, fonts, self.theme)
 
         if nname is not None:
             try:
@@ -349,11 +375,11 @@ class Convert(object):
     def write(self, doc):
         toc = create_toc(doc, self.body, self.resolved_link_map, self.styles, self.object_map, self.log, self.namespace)
         raw = html.tostring(self.html, encoding='utf-8', doctype='<!DOCTYPE html>')
-        with open(os.path.join(self.dest_dir, 'index.html'), 'wb') as f:
+        with lopen(os.path.join(self.dest_dir, 'index.html'), 'wb') as f:
             f.write(raw)
         css = self.styles.generate_css(self.dest_dir, self.docx, self.notes_nopb, self.nosupsub)
         if css:
-            with open(os.path.join(self.dest_dir, 'docx.css'), 'wb') as f:
+            with lopen(os.path.join(self.dest_dir, 'docx.css'), 'wb') as f:
                 f.write(css.encode('utf-8'))
 
         opf = OPFCreator(self.dest_dir, self.mi)
@@ -365,12 +391,13 @@ class Convert(object):
         opf.create_spine(['index.html'])
         if self.cover_image is not None:
             opf.guide.set_cover(self.cover_image)
+
         def process_guide(E, guide):
             if self.toc_anchor is not None:
                 guide.append(E.reference(
                     href='index.html#' + self.toc_anchor, title=_('Table of Contents'), type='toc'))
         toc_file = os.path.join(self.dest_dir, 'toc.ncx')
-        with open(os.path.join(self.dest_dir, 'metadata.opf'), 'wb') as of, open(toc_file, 'wb') as ncx:
+        with lopen(os.path.join(self.dest_dir, 'metadata.opf'), 'wb') as of, open(toc_file, 'wb') as ncx:
             opf.render(of, ncx, 'toc.ncx', process_guide=process_guide)
         if os.path.getsize(toc_file) == 0:
             os.remove(toc_file)
@@ -460,13 +487,17 @@ class Convert(object):
                     for a, t in tuple(self.anchor_map.iteritems()):
                         if t == old_anchor:
                             self.anchor_map[a] = current_anchor
+        if current_anchor is not None:
+            # This paragraph had no <w:r> descendants
+            dest.set('id', current_anchor)
+            current_anchor = None
 
         m = re.match(r'heading\s+(\d+)$', style.style_name or '', re.IGNORECASE)
         if m is not None:
             n = min(6, max(1, int(m.group(1))))
             dest.tag = 'h%d' % n
 
-        if style.direction == 'rtl':
+        if style.bidi is True:
             dest.set('dir', 'rtl')
 
         border_runs = []
@@ -674,6 +705,15 @@ class Convert(object):
             lang = html_lang(style.lang)
             if lang is not None and lang != self.doc_lang:
                 ans.set('lang', lang)
+        if style.rtl is True:
+            ans.set('dir', 'rtl')
+        if is_symbol_font(style.font_family):
+            for elem in text:
+                if elem.text:
+                    elem.text = map_symbol_text(elem.text, style.font_family)
+                if elem.tail:
+                    elem.tail = map_symbol_text(elem.tail, style.font_family)
+            style.font_family = 'sans-serif'
         return ans
 
     def add_frame(self, html_obj, style):
@@ -707,9 +747,32 @@ class Convert(object):
         rmap = {v:k for k, v in self.object_map.iteritems()}
         for border_style, blocks in self.block_runs:
             paras = tuple(rmap[p] for p in blocks)
+            for p in paras:
+                if p.tag == 'li':
+                    has_li = True
+                    break
+            else:
+                has_li = False
             parent = paras[0].getparent()
-            idx = parent.index(paras[0])
-            frame = DIV(*paras)
+            if parent.tag in ('ul', 'ol'):
+                ul = parent
+                parent = ul.getparent()
+                idx = parent.index(ul)
+                frame = DIV(ul)
+            elif has_li:
+                def top_level_tag(x):
+                    while True:
+                        q = x.getparent()
+                        if q is parent or q is None:
+                            break
+                        x = q
+                    return x
+                paras = tuple(map(top_level_tag, paras))
+                idx = parent.index(paras[0])
+                frame = DIV(*paras)
+            else:
+                idx = parent.index(paras[0])
+                frame = DIV(*paras)
             parent.insert(idx, frame)
             self.framed_map[frame] = css = border_style.css
             self.styles.register(css, 'frame')
@@ -758,6 +821,7 @@ class Convert(object):
         if len(run) > 1:
             process_run(run)
 
+
 if __name__ == '__main__':
     import shutil
     from calibre.utils.logging import default_log
@@ -767,4 +831,3 @@ if __name__ == '__main__':
         shutil.rmtree(dest_dir)
     os.mkdir(dest_dir)
     Convert(sys.argv[-1], dest_dir=dest_dir, log=default_log)()
-
