@@ -7,11 +7,12 @@ __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import string
-from future_builtins import map
+from polyglot.builtins import map
 
 from calibre.utils.config import JSONConfig
 from calibre.spell.dictionary import Dictionaries, parse_lang_code
 
+CONTAINER_DND_MIMETYPE = 'application/x-calibre-container-name-list'
 tprefs = JSONConfig('tweak_book_gui')
 d = tprefs.defaults
 
@@ -49,6 +50,7 @@ d['spell_check_case_sensitive_sort'] = False
 d['inline_spell_check'] = True
 d['custom_themes'] = {}
 d['remove_unused_classes'] = False
+d['merge_identical_selectors'] = False
 d['global_book_toolbar'] = [
 'new-file', 'open-book',  'save-book', None, 'global-undo', 'global-redo', 'create-checkpoint', None, 'donate', 'user-manual']
 d['global_tools_toolbar'] = [
@@ -57,7 +59,7 @@ d['global_tools_toolbar'] = [
 ]
 d['global_plugins_toolbar'] = []
 d['editor_common_toolbar'] = [('editor-' + x) if x else None for x in ('undo', 'redo', None, 'cut', 'copy', 'paste', 'smart-comment')]
-d['editor_css_toolbar'] = ['pretty-current', 'insert-image']
+d['editor_css_toolbar'] = ['pretty-current', 'editor-sort-css', 'insert-image']
 d['editor_xml_toolbar'] = ['pretty-current', 'insert-tag']
 d['editor_html_toolbar'] = ['fix-html-current', 'pretty-current', 'insert-image', 'insert-hyperlink', 'insert-tag', 'change-paragraph']
 d['editor_format_toolbar'] = [('format-text-' + x) if x else x for x in (
@@ -73,21 +75,30 @@ d['editor_accepts_drops'] = True
 d['toolbar_icon_size'] = 24
 d['insert_full_screen_image'] = False
 d['preserve_aspect_ratio_when_inserting_image'] = False
+d['file_list_shows_full_pathname'] = False
+d['auto_link_stylesheets'] = True
+d['check_external_link_anchors'] = True
 del d
 
 ucase_map = {l:string.ascii_uppercase[i] for i, l in enumerate(string.ascii_lowercase)}
+
+
 def capitalize(x):
     return ucase_map[x[0]] + x[1:]
 
+
 _current_container = None
 
+
 def current_container():
     return _current_container
 
+
 def set_current_container(container):
     global _current_container
     _current_container = container
 
+
 class NonReplaceDict(dict):
 
     def __setitem__(self, k, v):
@@ -95,6 +106,7 @@ class NonReplaceDict(dict):
             raise ValueError('The key %s is already present' % k)
         dict.__setitem__(self, k, v)
 
+
 actions = NonReplaceDict()
 editors = NonReplaceDict()
 toolbar_actions = NonReplaceDict()
@@ -104,11 +116,13 @@ editor_toolbar_actions = {
 TOP = object()
 dictionaries = Dictionaries()
 
+
 def editor_name(editor):
     for n, ed in editors.iteritems():
         if ed is editor:
             return n
 
+
 def set_book_locale(lang):
     dictionaries.initialize()
     try:
@@ -120,6 +134,7 @@ def set_book_locale(lang):
     from calibre.gui2.tweak_book.editor.syntax.html import refresh_spell_check_status
     refresh_spell_check_status()
 
+
 def verify_link(url, name=None):
     if _current_container is None or name is None:
         return None
@@ -135,9 +150,10 @@ def verify_link(url, name=None):
         return True
     return False
 
+
 def update_mark_text_action(ed=None):
     has_mark = False
     if ed is not None and ed.has_line_numbers:
         has_mark = bool(ed.selected_text) or not ed.has_marked_text
     ac = actions['mark-selected-text']
-    ac.setText(ac.default_text if has_mark else _('Unmark marked text'))
+    ac.setText(ac.default_text if has_mark else _('&Unmark marked text'))
